@@ -1,4 +1,4 @@
-# Copyright 1999-2012 Gentoo Foundation
+# Copyright 1999-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 from __future__ import print_function
@@ -169,6 +169,7 @@ def insert_optional_args(args):
 		'K' : y_or_n,
 		'q' : y_or_n,
 		'v' : y_or_n,
+		'w' : y_or_n,
 	}
 
 	arg_stack = args[:]
@@ -325,6 +326,11 @@ def parse_opts(tmpcmdline, silent=False):
 			"action":"store"
 		},
 
+		"--accept-restrict": {
+			"help":"temporarily override ACCEPT_RESTRICT",
+			"action":"store"
+		},
+
 		"--backtrack": {
 
 			"help"   : "Specifies how many times to backtrack if dependency " + \
@@ -608,6 +614,7 @@ def parse_opts(tmpcmdline, silent=False):
 		},
 
 		"--select": {
+			"shortopt" : "-w",
 			"help"    : "add specified packages to the world set " + \
 			            "(inverse of --oneshot)",
 			"type"    : "choice",
@@ -997,8 +1004,6 @@ def emerge_main(args=None):
 	if args is None:
 		args = sys.argv[1:]
 
-	portage._disable_legacy_globals()
-	portage._internal_warnings = True
 	# Disable color until we're sure that it should be enabled (after
 	# EMERGE_DEFAULT_OPTS has been parsed).
 	portage.output.havecolor = 0
@@ -1016,6 +1021,8 @@ def emerge_main(args=None):
 		os.environ["ROOT"] = myopts["--root"]
 	if "--accept-properties" in myopts:
 		os.environ["ACCEPT_PROPERTIES"] = myopts["--accept-properties"]
+	if "--accept-restrict" in myopts:
+		os.environ["ACCEPT_RESTRICT"] = myopts["--accept-restrict"]
 
 	# optimize --help (no need to load config / EMERGE_DEFAULT_OPTS)
 	if myaction == "help":
@@ -1036,7 +1043,8 @@ def emerge_main(args=None):
 
 	tmpcmdline = []
 	if "--ignore-default-opts" not in myopts:
-		tmpcmdline.extend(settings["EMERGE_DEFAULT_OPTS"].split())
+		tmpcmdline.extend(portage.util.shlex_split(
+			settings.get("EMERGE_DEFAULT_OPTS", "")))
 	tmpcmdline.extend(args)
 	myaction, myopts, myfiles = parse_opts(tmpcmdline)
 
