@@ -58,7 +58,9 @@ DEPCACHE_PATH            = "/var/cache/edb/dep"
 GLOBAL_CONFIG_PATH       = "/usr/share/portage/config"
 
 # these variables are not used with target_root or config_root
-PORTAGE_BASE_PATH        = os.path.join(os.sep, os.sep.join(__file__.split(os.sep)[:-3]))
+# NOTE: Use realpath(__file__) so that python module symlinks in site-packages
+# are followed back to the real location of the whole portage installation.
+PORTAGE_BASE_PATH        = os.path.join(os.sep, os.sep.join(os.path.realpath(__file__.rstrip("co")).split(os.sep)[:-3]))
 PORTAGE_BIN_PATH         = PORTAGE_BASE_PATH + "/bin"
 PORTAGE_PYM_PATH         = PORTAGE_BASE_PATH + "/pym"
 LOCALE_DATA_PATH         = PORTAGE_BASE_PATH + "/locale"  # FIXME: not used
@@ -163,13 +165,15 @@ if "PORTAGE_OVERRIDE_EPREFIX" in os.environ:
 	if EPREFIX:
 		EPREFIX = os.path.normpath(EPREFIX)
 
+VCS_DIRS = ("CVS", "RCS", "SCCS", ".bzr", ".git", ".hg", ".svn")
+
 # ===========================================================================
 # END OF CONSTANTS -- END OF CONSTANTS -- END OF CONSTANTS -- END OF CONSTANT
 # ===========================================================================
 
 # Private constants for use in conditional code in order to minimize the diff
 # between branches.
-_DEPCLEAN_LIB_CHECK_DEFAULT = 'n'
+_DEPCLEAN_LIB_CHECK_DEFAULT = True
 _ENABLE_REPO_NAME_WARN  = False
 _ENABLE_SET_CONFIG      = False
 _ENABLE_INHERIT_CHECK   = False
