@@ -1,6 +1,8 @@
-# Copyright 1998-2012 Gentoo Foundation
+# Copyright 1998-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
+from __future__ import unicode_literals
+
 __all__ = ["bindbapi", "binarytree"]
 
 import portage
@@ -41,6 +43,7 @@ import subprocess
 import sys
 import tempfile
 import textwrap
+import traceback
 import warnings
 from gzip import GzipFile
 from itertools import chain
@@ -138,15 +141,15 @@ class bindbapi(fakedbapi):
 			if myval:
 				mydata[x] = " ".join(myval.split())
 
-		if not mydata.setdefault('EAPI', _unicode_decode('0')):
-			mydata['EAPI'] = _unicode_decode('0')
+		if not mydata.setdefault('EAPI', '0'):
+			mydata['EAPI'] = '0'
 
 		if cache_me:
 			aux_cache = self._aux_cache_slot_dict()
 			for x in self._aux_cache_keys:
-				aux_cache[x] = mydata.get(x, _unicode_decode(''))
+				aux_cache[x] = mydata.get(x, '')
 			self._aux_cache[mycpv] = aux_cache
-		return [mydata.get(x, _unicode_decode('')) for x in wants]
+		return [mydata.get(x, '') for x in wants]
 
 	def aux_update(self, cpv, values):
 		if not self.bintree.populated:
@@ -258,7 +261,7 @@ def _pkgindex_cpv_map_latest_build(pkgindex):
 
 class binarytree(object):
 	"this tree scans for a list of all packages available in PKGDIR"
-	def __init__(self, _unused=None, pkgdir=None,
+	def __init__(self, _unused=DeprecationWarning, pkgdir=None,
 		virtual=DeprecationWarning, settings=None):
 
 		if pkgdir is None:
@@ -267,11 +270,11 @@ class binarytree(object):
 		if settings is None:
 			raise TypeError("settings parameter is required")
 
-		if _unused is not None and _unused != settings['ROOT']:
-			warnings.warn("The root parameter of the "
+		if _unused is not DeprecationWarning:
+			warnings.warn("The first parameter of the "
 				"portage.dbapi.bintree.binarytree"
-				" constructor is now unused. Use "
-				"settings['ROOT'] instead.",
+				" constructor is now unused. Instead "
+				"settings['ROOT'] is used.",
 				DeprecationWarning, stacklevel=2)
 
 		if virtual is not DeprecationWarning:
@@ -304,15 +307,15 @@ class binarytree(object):
 			self._pkgindex_aux_keys = \
 				["BUILD_TIME", "CHOST", "DEPEND", "DESCRIPTION", "EAPI",
 				"HDEPEND", "IUSE", "KEYWORDS", "LICENSE", "PDEPEND", "PROPERTIES",
-				"PROVIDE", "RDEPEND", "repository", "SLOT", "USE", "DEFINED_PHASES",
+				"PROVIDE", "RESTRICT", "RDEPEND", "repository", "SLOT", "USE", "DEFINED_PHASES",
 				"BASE_URI"]
 			self._pkgindex_aux_keys = list(self._pkgindex_aux_keys)
 			self._pkgindex_use_evaluated_keys = \
 				("DEPEND", "HDEPEND", "LICENSE", "RDEPEND",
-				"PDEPEND", "PROPERTIES", "PROVIDE")
+				"PDEPEND", "PROPERTIES", "PROVIDE", "RESTRICT")
 			self._pkgindex_header_keys = set([
 				"ACCEPT_KEYWORDS", "ACCEPT_LICENSE",
-				"ACCEPT_PROPERTIES", "CBUILD",
+				"ACCEPT_PROPERTIES", "ACCEPT_RESTRICT", "CBUILD",
 				"CONFIG_PROTECT", "CONFIG_PROTECT_MASK", "FEATURES",
 				"GENTOO_MIRRORS", "INSTALL_MASK", "SYNC", "USE"])
 			self._pkgindex_default_pkg_data = {
@@ -561,6 +564,20 @@ class binarytree(object):
 			if not os.path.isdir(path):
 				raise
 
+	def _file_permissions(self, path):
+		try:
+			pkgdir_st = os.stat(self.pkgdir)
+		except OSError:
+			pass
+		else:
+			pkgdir_gid = pkgdir_st.st_gid
+			pkgdir_grp_mode = 0o0060 & pkgdir_st.st_mode
+			try:
+				portage.util.apply_permissions(path, gid=pkgdir_gid,
+					mode=pkgdir_grp_mode, mask=0)
+			except PortageException:
+				pass
+
 	def _move_to_all(self, cpv):
 		"""If the file exists, move it.  Whether or not it exists, update state
 		for future getname() calls."""
@@ -868,46 +885,76 @@ class binarytree(object):
 				# protocols and requires the base url to have a trailing
 				# slash, so join manually...
 				url = base_url.rstrip("/") + "/Packages"
-				try:
-					f = _urlopen(url, if_modified_since=local_timestamp)
-					if hasattr(f, 'headers') and f.headers.get('timestamp', ''):
-						remote_timestamp = f.headers.get('timestamp')
-				except IOError as err:
-					if hasattr(err, 'code') and err.code == 304: # not modified (since local_timestamp)
-						raise UseCachedCopyOfRemoteIndex()
+				f = None
+
+				# Don't use urlopen for https, since it doesn't support
+				# certificate/hostname verification (bug #469888).
+				if parsed_url.scheme not in ('https',):
+					try:
+						f = _urlopen(url, if_modified_since=local_timestamp)
+						if hasattr(f, 'headers') and f.headers.get('timestamp', ''):
+							remote_timestamp = f.headers.get('timestamp')
+					except IOError as err:
+						if hasattr(err, 'code') and err.code == 304: # not modified (since local_timestamp)
+							raise UseCachedCopyOfRemoteIndex()
+
+						if parsed_url.scheme in ('ftp', 'http', 'https'):
+							# This protocol is supposedly supported by urlopen,
+							# so apparently there's a problem with the url
+							# or a bug in urlopen.
+							if self.settings.get("PORTAGE_DEBUG", "0") != "0":
+								traceback.print_exc()
+
+							raise
+
+				if f is None:
 
 					path = parsed_url.path.rstrip("/") + "/Packages"
 
-					if parsed_url.scheme == 'sftp':
-						# The sftp command complains about 'Illegal seek' if
-						# we try to make it write to /dev/stdout, so use a
-						# temp file instead.
-						fd, tmp_filename = tempfile.mkstemp()
-						os.close(fd)
+					if parsed_url.scheme == 'ssh':
+						# Use a pipe so that we can terminate the download
+						# early if we detect that the TIMESTAMP header
+						# matches that of the cached Packages file.
+						ssh_args = ['ssh']
 						if port is not None:
-							port_args = ['-P', "%s" % (port,)]
-						proc = subprocess.Popen(['sftp'] + port_args + \
-							[user_passwd + host + ":" + path, tmp_filename])
-						if proc.wait() != os.EX_OK:
-							raise
-						f = open(tmp_filename, 'rb')
-					elif parsed_url.scheme == 'ssh':
-						if port is not None:
-							port_args = ['-p', "%s" % (port,)]
-						proc = subprocess.Popen(['ssh'] + port_args + \
-							[user_passwd + host, '--', 'cat', path],
+							ssh_args.append("-p%s" % (port,))
+						# NOTE: shlex evaluates embedded quotes
+						ssh_args.extend(portage.util.shlex_split(
+							self.settings.get("PORTAGE_SSH_OPTS", "")))
+						ssh_args.append(user_passwd + host)
+						ssh_args.append('--')
+						ssh_args.append('cat')
+						ssh_args.append(path)
+
+						proc = subprocess.Popen(ssh_args,
 							stdout=subprocess.PIPE)
 						f = proc.stdout
 					else:
 						setting = 'FETCHCOMMAND_' + parsed_url.scheme.upper()
 						fcmd = self.settings.get(setting)
 						if not fcmd:
-							raise
+							fcmd = self.settings.get('FETCHCOMMAND')
+							if not fcmd:
+								raise EnvironmentError("FETCHCOMMAND is unset")
+
 						fd, tmp_filename = tempfile.mkstemp()
 						tmp_dirname, tmp_basename = os.path.split(tmp_filename)
 						os.close(fd)
-						success = portage.getbinpkg.file_get(url,
-						     tmp_dirname, fcmd=fcmd, filename=tmp_basename)
+
+						fcmd_vars = {
+							"DISTDIR": tmp_dirname,
+							"FILE": tmp_basename,
+							"URI": url
+						}
+
+						for k in ("PORTAGE_SSH_OPTS",):
+							try:
+								fcmd_vars[k] = self.settings[k]
+							except KeyError:
+								pass
+
+						success = portage.getbinpkg.file_get(
+							fcmd=fcmd, fcmd_vars=fcmd_vars)
 						if not success:
 							raise EnvironmentError("%s failed" % (setting,))
 						f = open(tmp_filename, 'rb')
@@ -1073,6 +1120,10 @@ class binarytree(object):
 				if not samefile:
 					self._ensure_dir(os.path.dirname(new_filename))
 					_movefile(filename, new_filename, mysettings=self.settings)
+				full_path = new_filename
+
+			self._file_permissions(full_path)
+
 			if self._all_directory and \
 				self.getname(cpv).split(os.path.sep)[-2] == "All":
 				self._create_symlink(cpv)
@@ -1145,6 +1196,7 @@ class binarytree(object):
 			f.close()
 			if f_close is not None:
 				f_close.close()
+			self._file_permissions(fname)
 			# some seconds might have elapsed since TIMESTAMP
 			os.utime(fname, (atime, mtime))
 
@@ -1356,19 +1408,14 @@ class binarytree(object):
 				f.close()
 		return pkgindex
 
-	def digestCheck(self, pkg):
-		"""
-		Verify digests for the given package and raise DigestException
-		if verification fails.
-		@rtype: bool
-		@return: True if digests could be located, False otherwise.
-		"""
-		cpv = pkg
-		if not isinstance(cpv, basestring):
-			cpv = pkg.cpv
-			pkg = None
+	def _get_digests(self, pkg):
 
-		pkg_path = self.getname(cpv)
+		try:
+			cpv = pkg.cpv
+		except AttributeError:
+			cpv = pkg
+
+		digests = {}
 		metadata = None
 		if self._remotepkgs is None or cpv not in self._remotepkgs:
 			for d in self._load_pkgindex().packages:
@@ -1378,9 +1425,8 @@ class binarytree(object):
 		else:
 			metadata = self._remotepkgs[cpv]
 		if metadata is None:
-			return False
+			return digests
 
-		digests = {}
 		for k in hashfunc_map:
 			v = metadata.get(k)
 			if not v:
@@ -1394,9 +1440,27 @@ class binarytree(object):
 				writemsg(_("!!! Malformed SIZE attribute in remote " \
 				"metadata for '%s'\n") % cpv)
 
+		return digests
+
+	def digestCheck(self, pkg):
+		"""
+		Verify digests for the given package and raise DigestException
+		if verification fails.
+		@rtype: bool
+		@return: True if digests could be located, False otherwise.
+		"""
+
+		digests = self._get_digests(pkg)
+
 		if not digests:
 			return False
 
+		try:
+			cpv = pkg.cpv
+		except AttributeError:
+			cpv = pkg
+
+		pkg_path = self.getname(cpv)
 		hash_filter = _hash_filter(
 			self.settings.get("PORTAGE_CHECKSUM_FILTER", ""))
 		if not hash_filter.transparent:
