@@ -164,6 +164,9 @@ class EbuildQuote(LineCheck):
 		"GAMES_DATADIR_BASE", "GAMES_SYSCONFDIR", "GAMES_STATEDIR",
 		"GAMES_LOGDIR", "GAMES_BINDIR"]
 
+	# variables for multibuild.eclass
+	var_names += ["BUILD_DIR"]
+
 	var_names = "(%s)" % "|".join(var_names)
 	var_reference = re.compile(r'\$(\{'+var_names+'\}|' + \
 		var_names + '\W)')
@@ -381,6 +384,7 @@ class InheritDeprecated(LineCheck):
 	deprecated_classes = {
 		"bash-completion": "bash-completion-r1",
 		"boost-utils": False,
+		"distutils": "distutils-r1",
 		"gems": "ruby-fakegem",
 		"git": "git-2",
 		"mozconfig-2": "mozconfig-3",
@@ -388,6 +392,7 @@ class InheritDeprecated(LineCheck):
 		"php-ext-pecl-r1": "php-ext-pecl-r2",
 		"php-ext-source-r1": "php-ext-source-r2",
 		"php-pear": "php-pear-r1",
+		"python": "python-r1 / python-single-r1 / python-any-r1",
 		"python-distutils-ng": "python-r1 + distutils-r1",
 		"qt3": False,
 		"qt4": "qt4-r2",
@@ -593,7 +598,8 @@ _eclass_info = {
 		),
 
 		# These are "eclasses are the whole ebuild" type thing.
-		'exempt_eclasses': _eclass_export_functions + ('autotools', 'libtool'),
+		'exempt_eclasses': _eclass_export_functions + ('autotools', 'libtool',
+			'multilib-minimal'),
 
 		'comprehensive': False
 	},
